% function PerfResults4Java()
%
% Evaluating results of classifiers from Java
% 
%
% Copyright Peng Chen @ King Abdullah University of Science and Technology (KAUST)
% $Revision: 1.0 $   $Date: Apr 13, 2012 10:12:00 $

%% extract data from csv files. the last column is of class labels, 
%% and other columns are for each classifier with corresponding AAindex1 vector
%% extract inform from java results
winLen = 11;
ProbEachFeature.IBk = importdata(['ProbEachFeatureIBk' num2str(winLen)  '.csv']);

%% perform all combinations of feature sets and find the optimal combinations.
numField = 1;
names = fieldnames(ProbEachFeature);
data = ProbEachFeature.(genvarname(names{numField,1})).data;
T_test0 = data(:,size(data,2));
Decision = data(:,1:size(data,2)-1); % for initial setup
PreLabels = zeros(size(Decision,1),1);
num1 = 1;
num2 = 1;
num3 = 1;
num4 = 1;
Result_IBk_F1 = cell(5000000,8);
Result_IBk_Acc = cell(5000000,8);
Result_IBk_CC = cell(5000000,8);
Result_IBk_All = cell(5000000,8);
for i = 1 : 5%size(Decision, 2) % the size of combination of feature sets
    y = 1 : i;
    n = size(Decision, 2);
    m = i;
    % disp(y);
    while any(y~=n-m+1:n)
        j=m;
        while y(j)>=n-m+j
          j=j-1;
        end
        h=j;
        y(h)=y(h)+1;
        for j=h+1:m
          y(j)=y(j-1)+1;
        end
        
        preComb = zeros(size(Decision,1),1);
        for k = 1 : length(y)   
            temp = ProbEachFeature.(genvarname(names{numField,1})).data(:,y(k));
            decision = sort(temp);
            PreLabels(temp>temp(ceil(length(decision)*0.5)),1)=1;
            PreLabels(temp<=temp(ceil(length(decision)*0.5)),1)=0;        
            preComb = preComb + PreLabels;
        end
        for l = 1 : max(preComb)
            PreLabels(preComb>=l)=1;
            PreLabels(preComb<l)=0;
            [Sen, Spec, Acc, CC, Precision, Fmeasure] = Performance(T_test0', PreLabels');
            % the cutoffs below should be changed with different cases
            if Fmeasure > 0.72 && CC > 0 
                fprintf(1, 'Combination = ');
                fprintf(1, '%2d ', y(:));
                fprintf(1, ' l=%-2d Sen=%2.4f Spec=%2.4f Acc=%2.4f CC=%2.4f Precision=%2.4f Fmeasure=%2.4f\n', l, Sen, Spec, Acc, CC, Precision, Fmeasure);
            end
            if Fmeasure > 0.6 && CC > 0
                Result_IBk_F1(num1, :) = {y, l, Sen, Spec, Acc, CC, Precision, Fmeasure};
                num1 = num1 + 1;
            end
            if Acc > 0.66 && CC > 0
                Result_IBk_Acc(num2, :) = {y, l, Sen, Spec, Acc, CC, Precision, Fmeasure};
                num2 = num2 + 1;
            end
            if CC > 0.25 && Sen*Spec > 0
                Result_IBk_CC(num3, :) = {y, l, Sen, Spec, Acc, CC, Precision, Fmeasure};
                num3 = num3 + 1;
            end
            if CC > 0.25 && Acc > 0.66 && Fmeasure > 0.60
                Result_IBk_All(num4, :) = {y, l, Sen, Spec, Acc, CC, Precision, Fmeasure};
                num4 = num4 + 1;
            end
        end
        clear preComb PreLabels;
    end
end

%% below for the calculation of mean performance
for i = 1 : size(Result_IBk_All,1)
    temp(i) = Result_IBk_All{i,8}; % F1
end
[temp0, I] = sort(temp, 'descend');
prefer = zeros(132,1);
FAll = zeros(size(Result_IBk_All,1),6);
for i = 1 : size(Result_IBk_All,1)
    for j = 1 : length(Result_IBk_All{I(i),1})
        prefer(Result_IBk_All{I(i),1}(j),1) = prefer(Result_IBk_All{I(i),1}(j),1) +1;
    end
    for j = 1 : size(Result_IBk_All,2)-2
        FAll(i,j) = Result_IBk_All{I(i),j+2};
    end
end
meanFAll = mean(FAll);
FpreferAll = prefer/sum(prefer);
[C IFpreferAll] = sort(FpreferAll, 'descend');
clear temp temp0 C I

%% achieve the best predictions for the test set
top = IFpreferAll(1:10);%[112 65 127 66 81 40 4 52 89 86];
T_test0 = ProbEachFeature.IBk.data(:,size(ProbEachFeature.IBk.data,2));
Decision = ProbEachFeature.IBk.data(:,top); % for initial setup
for k = 50 % for roc
    preComb = zeros(size(Decision,1),1);
    for numTop = 1 : length(top)
        temp = Decision(:,numTop);
        decision = sort(temp);
        PreLabels(temp>temp(ceil(length(decision)*k/100)),1)=1;
        PreLabels(temp<=temp(ceil(length(decision)*k/100)),1)=0;        
        preComb = preComb + PreLabels;
    end
    for l = 3%1 : max(preComb)
        PreLabels(preComb>=l)=1;
        PreLabels(preComb<l)=0;
        [Sen, Spec, Acc, CC, Precision, Fmeasure] = Performance(T_test0', PreLabels');
        Prediction4testIBk10 = PreLabels;
    end
    clear preComb;
end

