/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 */

/*
 *    Performance.java
 *    Copyright (C) Peng Chen, NTU, 9:40 Apr 3, 2010
 *
 */
package pchen.classifiers;

public class Performance {
	private double tp;
	private double tn;
	private double fp;
	private double fn;

	public Performance() {

	}

	/**
	 * Count the number of tp, fp, tn, or fn
	 * 
	 * @param predClassCombiner
	 *            Class prediction for classifier.
	 * @param testClasses
	 *            True class list for classifier.
	 * @return null
	 */
	public Performance(double[] predClassCombiner, double[] testClasses) {
		for (int i = 0; i < predClassCombiner.length; i++) {
			if (testClasses[i] == 1 && predClassCombiner[i] == 1) {
				tp = tp + 1;
			} else if (testClasses[i] == 0 && predClassCombiner[i] == 1) {
				fp = fp + 1;
			} else if (testClasses[i] == 0 && predClassCombiner[i] == 0) {
				tn = tn + 1;
			} else if (testClasses[i] == 1 && predClassCombiner[i] == 0) {
				fn = fn + 1;
			}
		}
	}

	/**
	 * Get recall
	 * 
	 * @return recall
	 */
	public double recall() {
		double total = (tp + fn);
		if (total == 0) {
			return 0;
		}
		return tp / total;
	}

	/**
	 * Get specificity
	 * 
	 * @return specificity
	 */
	public double specificity() {
		double total = (tn + fp);
		if (total == 0) {
			return 0;
		}
		return tn / total;
	}

	/**
	 * Get accuracy
	 * 
	 * @return accuracy
	 */
	public double accuracy() {
		double total = tp + fp + tn + fn;
		if (total == 0) {
			return 0;
		}
		return (tn + tp) / total;
	}

	/**
	 * Get correlation coefficient
	 * 
	 * @return correlationCoefficient
	 */
	public double correlationCoefficient() {
		double total = (Math
				.sqrt((tp + fp) * (tp + fn) * (tn + fp) * (tn + fn)));
		if (total == 0) {
			return 0;
		}
		return (tp * tn - fp * fn) / total;
	}

	/**
	 * Get precision
	 * 
	 * @return precision
	 */
	public double precision() {
		double total = (tp + fp);
		if (total == 0) {
			return 0;
		}
		return tp / total;
	}

	/**
	 * Get fMeasure
	 * 
	 * @return fMeasure
	 */
	public double fMeasure() {
		double total = (precision() + recall());
		if (total == 0) {
			return 0;
		}
		return 2 * precision() * recall() / total;
	}
}
