/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 */

/*
 *    Blast.java
 *    Copyright (C) 2012 Peng Chen, King Abdullah University of Science and 
 *    Technology, King of Saudi Arabia
 *
 */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.biojava.bio.BioException;
import org.biojava.bio.program.sax.blastxml.BlastXMLParserFacade;
import org.biojava.bio.program.ssbind.BlastLikeSearchBuilder;
import org.biojava.bio.program.ssbind.SeqSimilarityAdapter;
import org.biojava.bio.search.SearchContentHandler;
import org.biojava.bio.search.SeqSimilaritySearchHit;
import org.biojava.bio.search.SeqSimilaritySearchResult;
import org.biojava.bio.search.SeqSimilaritySearchSubHit;
import org.biojava.bio.seq.db.DummySequenceDB;
import org.biojava.bio.seq.db.DummySequenceDBInstallation;
import org.biojava.bio.symbol.Alignment;
import org.biojava.bio.symbol.LocationTools;
import org.biojavax.SimpleNamespace;
import org.biojavax.bio.alignment.blast.RemoteQBlastAlignmentProperties;
import org.biojavax.bio.alignment.blast.RemoteQBlastOutputFormat;
import org.biojavax.bio.alignment.blast.RemoteQBlastOutputProperties;
import org.biojavax.bio.alignment.blast.RemoteQBlastService;
import org.biojavax.bio.seq.RichSequence;
import org.biojavax.bio.seq.RichSequenceIterator;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class Blast {

	private static ArrayList<String> querySeq = new ArrayList<String>();
	private static ArrayList<String> queryName = new ArrayList<String>();
	static String inputSeqAdd;
	static String outputSaveAdd;
	final static String aa1name = "ACDEFGHIKLMNPQRSTVWYXXX-";
	final static String aa3name = "ALA CYS ASP GLU PHE GLY HIS ILE LYS LEU MET ASN PRO GLN ARG SER THR VAL TRP TYR PYL SEC XXX ---";

	/**
	 * Obtain remote BLAST file and save into a file
	 * 
	 * @param inputSeqAdd
	 *            Address of sequence file in FASTA format.
	 * @param outputSaveAdd
	 *            A 2D array for test inputs.
	 * @return null
	 * @method Parse sequences file, get each sequence of chain, get remote
	 *         BLAST results of the sequence, and finally save into a text file.
	 * @throws
	 */
	public Blast(String inputSeqAdd, String outputSaveAdd) {
		// try {
		// RemoteQBlastService blast = new RemoteQBlastService();
		// blast.setSequence(query);
		// blast.setDatabase(db);
		// blast.setProgram(program);
		// blast.setQBlastOutputFormat(0);
		// blast.executeSearch();
		// blast.getAlignmentResults();
		// blast.printRemoteBlastInfo();
		// } catch (BioException e) {
		// e.printStackTrace();
		// }
		// System.out.println(query);

		RemoteQBlastService rbw;
		RemoteQBlastOutputProperties rof;
		InputStream is;
		ArrayList<String> rid = new ArrayList<String>();
		String request = "";

		try {
			rbw = new RemoteQBlastService();
			SimpleNamespace ns = new SimpleNamespace("bj_blast");
			RichSequenceIterator rs = RichSequence.IOTools.readFastaProtein(
					new BufferedReader(new FileReader(inputSeqAdd)), ns);

			// You would imagine that one would blast a bunch of sequences of
			// identical nature with identical parameters...

			RemoteQBlastAlignmentProperties rqb = new RemoteQBlastAlignmentProperties();
			rqb.setBlastProgram("blastp");
			rqb.setBlastDatabase("nr");
			rqb.setAdvancedOptions("");

			// First, let's send all the sequences to the QBlast service and
			// keep the RID for fetching the results at some later moments
			// (actually, in a few seconds :-))
			//
			// Using a data structure to keep track of all request IDs is a good
			// practice.

			while (rs.hasNext()) {
				RichSequence rr = rs.nextRichSequence();
				queryName.add(rr.getName());
				querySeq.add(rr.seqString());
				request = rbw.sendAlignmentRequest(rr, rqb);
				rid.add(request);
				System.out.println(rr.getName());
			}

			// Let's check that our requests have been processed. If completed,
			// let's look at the alignments with my own selection of output and
			// alignment formats.
			System.out.println("Starting the blast ...");
			int numSeq = 0;
			for (String aRid : rid) {
				// System.out.println("trying to get BLAST results for RID "+
				// aRid);
				boolean wasBlasted = false;

				while (!wasBlasted) {
					wasBlasted = rbw.isReady(aRid, System.currentTimeMillis());
				}

				rof = new RemoteQBlastOutputProperties();
				rof.setOutputFormat(RemoteQBlastOutputFormat.XML);
				rof.setAlignmentOutputFormat(RemoteQBlastOutputFormat.PAIRWISE);
				rof.setDescriptionNumber(10);
				rof.setAlignmentNumber(10);

				// Simply to show you that your output options were followed
				// Set<String> test = rof.getOutputOptions();
				// for (String str : test)
				// System.out.println(str);

				is = rbw.getAlignmentResults(aRid, rof);
				BufferedReader br = new BufferedReader(
						new InputStreamReader(is));

				// write BLAST results into a file
				String line = null;
				try {
					BufferedWriter bw = new BufferedWriter(new FileWriter(
							outputSaveAdd + queryName.get(numSeq) + ".blast"));
					String str0 = "<Hsp_query-frame>";
					String str1 = "<Hsp_hit-frame>";
					while ((line = br.readLine()) != null) {
						if (!str0.regionMatches(false, 0, line, 6, 17)
								&& !str1.regionMatches(false, 0, line, 6, 15)) {
							bw.write(line);
							bw.newLine();
						}
					}
					bw.close();
				} catch (Exception e) {
				}
				// BlastParser(outputSaveAdd+"Blast\\"+queryName.get(numSeq)+".blast",
				// queryName.get(numSeq));
				System.out.println(queryName.get(numSeq));
				numSeq++;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (BioException bio) {
			bio.printStackTrace();
		}
	}

	public Blast() {
	}

	/**
	 * Parse BLAST results and save sequence profiles into a file
	 * 
	 * @param addrBlast
	 *            Address of BLAST file for one chain.
	 * @param queryName
	 *            Sequence description for remote BLAST.
	 * @param outputSaveAdd
	 *            File address to be saved.
	 * @return null
	 * @method
	 * @throws
	 */
	@SuppressWarnings("unchecked")
	public static void BlastParser(String addrBlast, String queryName,
			String outputSaveAdd) {
		try {
			// get the Blast input as a Stream
			InputStream is = new FileInputStream(new File(addrBlast));
			// make a BlastLikeSAXParser
			BlastXMLParserFacade parser = new BlastXMLParserFacade();
			// try to parse, even if the blast version is not recognized.
			// parser.setModeLazy();

			// make the SAX event adapter that will pass events to a Handler.
			SeqSimilarityAdapter adapter = new SeqSimilarityAdapter();

			// The list to hold the SeqSimilaritySearchResults
			List results = new ArrayList();

			// create the SearchContentHandler that will build
			// SeqSimilaritySearchResults in the results List
			SearchContentHandler builder = new BlastLikeSearchBuilder(results,
					new DummySequenceDB("queries"),
					new DummySequenceDBInstallation());

			// register builder with adapter
			adapter.setSearchContentHandler(builder);

			// set the parsers SAX event adapter
			parser.setContentHandler(adapter);

			// parse the file, after this the result List will be populated with
			// SeqSimilaritySearchResults
			parser.parse(new InputSource(is));

			// Blast2SeqProfile(results);
			Blast2SeqProfile(results, queryName, outputSaveAdd);
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (BioException e) {
			e.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Parse BLAST results to sequence profiles matrix and save the matrix into
	 * a file
	 * 
	 * @param results
	 *            BLAST results.
	 * @param queryName
	 *            Sequence description for remote BLAST.
	 * @param outputSaveAdd
	 *            File address to be saved.
	 * @return null
	 * @method
	 * @throws
	 */
	@SuppressWarnings("unchecked")
	private static void Blast2SeqProfile(List results, String queryName,
			String outputSaveAdd) {
		// output some blast details
		ArrayList alignment = new ArrayList();
		for (Iterator i = results.iterator(); i.hasNext();) {
			int numMSA = 0;
			SeqSimilaritySearchResult result = (SeqSimilaritySearchResult) i
					.next();
			/*
			 * Annotation anno = result.getAnnotation(); for (Iterator j =
			 * anno.keys().iterator(); j.hasNext(); ) { Object key = j.next();
			 * Object property = anno.getProperty(key);
			 * System.out.println(key+" : "+property); }
			 */

			// list the hits
			for (Iterator k = result.getHits().iterator(); k.hasNext();) {
				SeqSimilaritySearchHit hit = (SeqSimilaritySearchHit) k.next();
				String seqQuery = null;
				String seqSubject = null;
				for (Iterator l = hit.getSubHits().iterator(); l.hasNext();) {
					seqQuery = "";
					seqSubject = "";
					numMSA++;
					SeqSimilaritySearchSubHit subhit = (SeqSimilaritySearchSubHit) l
							.next();
					Alignment algn = subhit.getAlignment();
					String firstResidue = null;
					String secondResidue = null;
					for (int m = 0; m < subhit.getQueryStart() - 1; m++) {
						seqQuery = seqQuery + " ";
						seqSubject = seqSubject + " ";
					}
					// get the sub-alignment and obtain the subject sequence
					// strings
					for (int m = 0; m < algn.length(); m++) {
						Alignment subalgn = algn.subAlignment(null,
								LocationTools.makeLocation(m + 1, m + 1));
						String residuePair = subalgn.seqString();
						if (residuePair.length() == 9) {
							firstResidue = residuePair.substring(1, 4);
							secondResidue = residuePair.substring(5, 8);
						} else if (residuePair.length() > 9) {
							if (residuePair.regionMatches(false, 1, "[", 0, 1)) {
								if (residuePair.regionMatches(false, 2, "]", 0,
										1))// ([] [LYS TYR PYL CYS LEU SER PRO
									// HIS PHE ASN THR ASP MET GLN GLY
									// ILE VAL GLU ARG ALA SEC TRP])
									firstResidue = "---";
								else
									firstResidue = "XXX";
							} else
								firstResidue = residuePair.substring(1, 4);
							if (residuePair.regionMatches(false, residuePair
									.length() - 2, "]", 0, 1)) {
								if (residuePair.regionMatches(false,
										residuePair.length() - 3, "[", 0, 1))
									secondResidue = "---";
								else
									secondResidue = "XXX";
							} else
								firstResidue = residuePair.substring(
										residuePair.length() - 4, residuePair
												.length() - 1);
						} else {// "(ASP [])" or "([] ALA)"
							if (residuePair.regionMatches(false, residuePair
									.length() - 3, "[]", 0, 2)) {
								firstResidue = residuePair.substring(1, 4);
								secondResidue = "---";
							} else if (residuePair.regionMatches(false, 1,
									"[]", 0, 2)) {
								firstResidue = "---";
								secondResidue = residuePair.substring(4, 7);
							}
						}
						seqQuery = seqQuery + aa3To1(firstResidue);
						seqSubject = seqSubject + aa3To1(secondResidue);
						System.out.println(residuePair);
					}
					System.out.println(seqQuery);
					for (int m = seqQuery.length() - 1; m >= 0; m--) {
						if (seqQuery.substring(m, m + 1).equals("-")) {
							seqSubject = seqSubject.substring(0, m)
									+ seqSubject.substring(m + 1);
						}
					}
					System.out.println(seqSubject);

					if (numMSA == 1)
						alignment.add(0, seqQuery);
					alignment.add(numMSA, seqSubject);
				}
			}

			// modify the alignment List
			String[] strMSA = new String[alignment.size()];
			numMSA = 0;
			int lenQuery = 0;
			for (Iterator j = alignment.iterator(); j.hasNext();) {
				String str = (String) j.next().toString();
				strMSA[numMSA] = str;
				// get the maximum length of alignment sequence
				if (lenQuery < str.length())
					lenQuery = str.length();
				numMSA++;
			}

			// append necessary empty symbols to subject sequence strings
			for (int j = 0; j < strMSA.length; j++) {
				if (strMSA[j].length() < lenQuery) {
					for (int k = strMSA[j].length(); k < lenQuery; k++) {
						strMSA[j] = strMSA[j] + " ";
					}
				}
			}
			// get sequence profile
			boolean gap = false;
			double[][] seqProfile = SeqProfile(strMSA, gap);

			saveText(seqProfile, outputSaveAdd + queryName + ".sp");
			// BlastProfile seqProfile =
			// BlastProfile.createProfileResidues(strMSA);
		}
	}

	/**
	 * Save the matrix into a file
	 * 
	 * @param seqProfile
	 *            2D array for sequence profiles.
	 * @param fileFullAdd
	 *            File address to be saved.
	 * @return null
	 * @method
	 * @throws
	 */
	private static void saveText(double[][] seqProfile, String fileFullAdd) {
		try {
			FileOutputStream writer = new FileOutputStream(fileFullAdd);
			DataOutputStream osw = new DataOutputStream(writer);
			/*
			 * DecimalFormat df = new DecimalFormat("0.000"); // format a double
			 * value String num = df.format(a);
			 */
			for (int m = 0; m < seqProfile.length; m++) {
				for (int n = 0; n < seqProfile[0].length; n++) {
					osw.write((String.valueOf(seqProfile[m][n]) + " ")
							.getBytes());
				}
				osw.write("\r\n".getBytes()); // enter
			}
			osw.close();
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Parse multiple sequence alignment and save sequence profile into a file
	 * 
	 * @param strMSA
	 *            Alignment sequence strings
	 * @param gap
	 *            Whether to count the gap
	 * @return seqProfile: a 2D matrix (L*20)
	 * @method
	 * @throws
	 */
	private static double[][] SeqProfile(String[] strMSA, boolean gap) {
		int numHits = strMSA.length;
		int numResidues = strMSA[0].length();
		double[][] seqProfile = new double[numResidues][20];
		for (int i = 0; i < numResidues; i++) {
			String str4Residue = "";
			for (int j = 0; j < numHits; j++) {
				str4Residue = str4Residue + strMSA[j].charAt(i);
			}
			double numGapRemoved = 0;
			for (int k = 0; k < str4Residue.length(); k++) {
				if (!(str4Residue.substring(k, k + 1).equals(" "))
						&& !(str4Residue.substring(k, k + 1).equals("X"))) {
					numGapRemoved++;
				}
			}
			for (int j = 0; j < 20; j++) {
				double numPerResidue = 0;
				for (int k = 0; k < str4Residue.length(); k++) {
					if (str4Residue.charAt(k) == aa1name.charAt(j)) {
						numPerResidue++;
					}
					if (gap) {
						seqProfile[i][j] = numPerResidue / (double) numHits;
					} else {
						seqProfile[i][j] = numPerResidue / numGapRemoved;
					}
				}
			}
		}
		return seqProfile;
	}

	/**
	 * Convert amino acid with three letters to that with one letter
	 * 
	 * @param substring
	 *            string for three-letter residue names
	 * @return oneName string for one-letter residue
	 * @method
	 * @throws
	 */
	public static String aa3To1(String substring) {
		int posAA3Name = aa3name.indexOf(substring);
		String oneName = aa1name.substring((int) posAA3Name / 4,
				(int) posAA3Name / 4 + 1);
		return oneName;
	}
}
